@@ -109,6 +109,12 @@ C<< ( RESULT, COMMENT, HASH, EXPLAIN ) >>. RESULT is the result, e.g. "Fail",
 information about problem, mechanism for the Received-SPF header.
 EXPLAIN will be set to the explain string if RESULT is Fail.
 
+If no final result was achieved yet it will either return 
+C<< (undef,@QUERIES) >> with a list of new queries to continue, C<< ('') >>
+in case the ANSWER produced an error but got ignored, because there are
+other queries open, or C<< () >> in case the ANSWER was ignored because it
+did not match any open queries.
+
 =item mailheader
 
 Creates value for Received-SPF header based on the final answer from next().
@@ -195,7 +201,7 @@ use warnings;
 
 package Mail::SPF::Iterator;
 
-our $VERSION = '1.08';
+our $VERSION = '1.11';
 
 use fields (
 	# values given in or derived from params to new()
@@ -508,7 +514,8 @@ sub mailheader {
 # Returns: (undef,@dnsq) | ($status,$info,\%param,$explain) | ()
 #   (undef,@dnsq): @dnsq are more DNS questions
 #   ($status,$info,\%param,$explain): final response
-#   (): reply ignored, waiting for next reply
+#   (''): reply processed, but answer ignored (likely error)
+#   (): reply ignored, does not matching outstanding request
 ############################################################################
 sub next {
 	my Mail::SPF::Iterator $self = shift;
@@ -549,31 +556,29 @@ sub next {
 	my $found;
 	for (@$cb_queries) {
 		next if $qid != $_->{id}; # ID mismatch
+		next if $qtype ne $_->{q}->qtype;  # type mismatch
 
-		# presentation2wire
-		# $_->{q}->qname has still the raw (wire) value, because it was set to
-		# it but the qname of the response has the human readable presentation
-		# from the zonefiles :(
-		# fortunatly this applies only to DNS names with special chars
-		# see Net::DNS::wire2presentation
-		( my $qname = lc($question->qname) )
-			=~s{\\(?:(\d\d\d)|(.))}{ $2 || chr($1) }esg;
+		if ( lc($question->qname) eq lc($_->{q}->qname) ) {
+			$found = $_;
+			last;
+		}
 
-		if ( $qtype eq $_->{q}->qtype and $qname eq lc($_->{q}->qname)) {
+		# in case of special characters the names might have the 
+		# wire presentation \DDD or the raw presentation
+		# actual behavior depends on the Net::DNS version, so normalize
+		my $rname = lc($question->qname);
+		my $qname = lc($_->{q}->qname);
+		s{\\(?:(\d\d\d)|(.))}{ $2 || chr($1) }esg for($rname,$qname);
+		if ( $rname eq $qname ) {
 			$found = $_;
 			last;
 		}
 	}
 
 	if ( ! $found ) {
-		# unexpected response, type or domain do not match query -> TempError
-		my %want = map { $_->{q}->qtype => 1 } @$cb_queries;
-		my %name = map { $_->{q}->qname => 1 } @$cb_queries;
+		# packet does not match our queries
 		DEBUG( "found no open query for ".$question->string );
-		return ( SPF_TempError,
-			"getting ".join("|",keys %want)." for ".join("|",keys %name),
-			{ problem => "unexpected DNS response" },
-		);
+		return; # ignore problem
 	} elsif ( ++$found->{done} > 1 ) {
 		# duplicate response - ignore
 		DEBUG( "duplicate response, ignoring" );
@@ -585,9 +590,10 @@ sub next {
 	if ( $err ) {
 		if ( grep { ! $_->{done} } @$cb_queries ) {
 			# we still have outstanding queries, so we might still get answers
-			# -> return () as a sign, that we ignore this error
+			# -> return ('') as a sign, that we got an error to an outstanding
+			# trequest, but otherwise ignore this error
 			DEBUG( "ignore error '$err', we still have oustanding queries" );
-			return;
+			return ('');
 
 		} elsif ( my $r = $self->{result} ) {
 			# we have a final result already, so this error occured only while
@@ -725,7 +731,7 @@ sub _next_process_cbrv {
 	if ( $rv[0] eq SPF_Noop ) {
 		die "NOOP but no open queries"
 			if ! grep { ! $_->{done} } @{$self->{cbq}};
-		return;
+		return ('');
 	}
 
 	# inside include the response is only pre-final,
